/** This class, a controller for the GUI, makes the wavelet applet
 * @author Arthur Asuncion
 */

import javax.swing.*;


import java.awt.*;
import java.awt.event.*;

public class SignalControl extends JApplet{
	
	private SignalView view;
	JFrame frame;
	String sourcePath;
	
	Sound sound;
	
	
	/** Constructor
	 * @param f the JFrame where the GUI is created
	 */
	public SignalControl(JFrame f) {
		frame = f;
	}
	
	public SignalControl() {
		frame = new JFrame("AWE: Art's Wavelet Effects, by Arthur Asuncion");
	}	

	/** Initializes the applet
	 */
	public void init() {
		
		view = new SignalView(this);
		try {
			view.setLoadState();
		}
		catch(Exception e) {
			view.setMsg(e.toString());
			view.setLoadState();
		}
		
		
		// A listener for the load button
		view.addLoadListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {

				FileDialog fd = new FileDialog(frame);
				fd.setVisible(true);
				if (fd.getFile() == null) {return;}
				
				sourcePath = fd.getDirectory() + fd.getFile();
				
				try {
					sound = new Sound(sourcePath);
					view.setMsg(sourcePath + " sucessfully loaded");
					view.setStopState();
				}
				catch (Exception e) {
					view.setMsg("Load was unsuccessful. Try again.");
					view.setLoadState();
				}
				
			
			}
		});
		
		
		// A listener for the start button
		view.addStartListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				
				// Checks the user inputs in the GUI
				if (true) {
					//view.setStartState();
				
					try {	
						WaveletThread st = new WaveletThread(view);
						st.start();	
					}
					catch(Exception e) { // Catches unexpected any errors
						view.setMsg("Action cancelled due to an error.");
					}
				}
			}
		});
		
		
		// A listener for the stop button
		view.addStopListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				
				view.stop(true);
			}
		});
		
		// A listener for the pause button
		view.addPauseListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {

				view.pause(!view.isPaused());				
				view.setPauseState();
				view.setMsg(view.isPaused()?
							"Signal Paused...":"Signal Continued...");
			}
		});
		
		
	}
	
	
	/** This main method runs the applet
	 * @param String[] args no arguments needed
	 */
	
	public static void main(String[] args){
		SignalControl app = new SignalControl();
		app.init();
		app.start();
		app.getFrame().getContentPane().add(app);
		app.getFrame().pack();
		app.getFrame().setSize(880,600);
		//frame.setResizable(false);
		app.getFrame().setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		app.getFrame().setVisible(true);


	}

	JFrame getFrame() {
		return frame;
	}
	

	/** This thread class starts the Sound class. It is used so that
	 * the GUI can stay responsive.  It is also used so that the "pause" 
	 * and "stop" buttons can be clicked.
	 * @author Arthur Asuncion
	 */
	
	private class WaveletThread extends Thread {
	
		// data fields
		SignalView view;
		
		
		/** Constructor
		 * 
		 * @param v the SignalView
		 */
		WaveletThread(SignalView v) {
			view = v;
		}
		
		/** The method that is invoked when the thread is started
		 */
		public void run() {
			
			view.clearCanvas(1024 * 8);
			
			view.setMsg("Calculating Wavelet Transforms");
			sound.start(view);
			
			view.setMsg("Signal Ended");
			
			view.setStopState();
			
			return;
			
		}
	
	}
	
}
